#! /usr/bin/php -q
<?php

define('VERSION', '0.5');
define('DEFAULT_CONFIG_FILE', '/etc/rbackup.conf');
define('DEFAULT_CLASS_NAME', 'default');
define('RDIFF_BIN', 'rdiff-backup');

$quietMode = false;
$verboseMode = false;
$configurationFile = null;
$rbackupRoot = null;
$className = null;
$globalError = false;

function fame() {
  printf("rBackup version %-5s <toad+rbackup@lescigales.org>\n", VERSION);
}

$options = getopt("t:vhc:Vq");
foreach ($options as $option => $value) {
  if ($option == 'c') {
    $configurationFile = $value;
  }
  if ($option == 'v') {
    $verboseMode = true;
  }
  if ($option == 't') {
    $currentClassName = $value;
  }
  if ($option == 'V') {
    fame();
    exit(0);
  }
  if ($option == 'h') {
    syntax($argv);
    exit(0);
  }
  if ($option == 'q') {
    $quietMode = true;
  }
}

if ($quietMode) {
  ob_start();
}

fame();

if ($configurationFile == null) {
  $configurationFile = DEFAULT_CONFIG_FILE;
}

if ($currentClassName == null) {
  $currentClassName = DEFAULT_CLASS_NAME;
}

# Read entirely the configuration file
$fp = @fopen($configurationFile, "r") or 
	die("cannot open configuration file: $configurationFile\n");
$lineNumber = 0;
while ($fd = fgets($fp, 4096)) {
  $lineNumber++;
  $fd = trim($fd);
  /* Skip empty lines */
  if ($fd == null) {
    continue;
  }
  /* Skip comments */
  if (! strncmp($fd, "#", 1)) {
    continue;
  }
  $configurationTab[$lineNumber] = $fd;
}
fclose($fp);

# Then read the variables and rewrite the configuration file in memory
# by replacing variables by new values

$variablesList = array();
foreach($configurationTab as $lineNumber => $configLine)
{
#  echo $lineNumber, ": ", $configLine, "\n";
  $lineTab = explode(' ', $configLine);
  if ($lineTab[0] == 'var') {
    if (isset($lineTab[1]) && isset($lineTab[2])) {
      $varName = $lineTab[1];
      $varValue = $lineTab[2];
#      echo "variable spotted! $varName=$varValue\n";
      $variablesList[$varName] = $varValue;
    }
	
  }
}

krsort($variablesList);

# Replace all variables occurences:
foreach ($configurationTab as $lineNumber => $configLine)
{
  if (ereg('^var ', $configLine)) {
    unset($configurationTab[$lineNumber]);
    continue;
  }

  /* Not very efficient to do it line by line, but... */
  foreach ($variablesList as $varName => $varValue) {
#    echo "replacing $varName by $varValue\n";
    $configurationTab[$lineNumber] = 
      str_replace('$'.$varName, $varValue, $configurationTab[$lineNumber]);
#    echo "replaced: ", $configurationTab[$lineNumber], "\n";
  }
}

#foreach ($configurationTab as $n => $v) {
#  echo "ii $n: $v\n";
#}

$configOptions = array(
		       array("rbackup_root", 2, "set_rbackup_root"),
		       array("backup", 3, "backup"),
		       array("script", 3, "script"),
		       array('svnsync', 3, "svnsync"),
		       array("class", 2, "set_class")
		       );

# Run twice, once to set up global configuration values
# second time to actually run the backup
for ($configurationParsing = true; ; $configurationParsing = false) {
  foreach($configurationTab as $lineNumber => $configLine) {
    if ($verboseMode && $configurationParsing) {
      echo "parsing line $lineNumber: $configLine\n";
    }
    $lineTab = explode(' ', $configLine);
    $len = sizeof($lineTab);
    
    $selected = -1;
    for ($i = 0; $configOptions[$i]; $i++)
      {
	if (! strcmp($lineTab[0], $configOptions[$i][0])) {
	  $selected = $i;
	  break;
	}
      }
    if ($selected == -1) {
      echo "Unknown option `".$lineTab[0]."' at line $lineNumber\n";
      $errorConfig = true;
    }
    if ($len < $configOptions[$selected][1]) {
      echo "Not enough parameters at line $lineNumber\n";
      $errorConfig = true;
    }
    if ($len > $configOptions[$selected][1]) {
      echo "Too many parameters at line $lineNumber\n";
      $errorConfig = true;
    }
    
    if ($errorConfig) {
      die("line $lineNumber: $configLine\n");
    }
    
    /* If quietMode is On, clean and bufferize the output */
    if ($quietMode == true) {
      /* Clean the old buffer if any */
      ob_clean();
      /* Restart the buffering */
      //      ob_start();
      /* Reset the error flag */
      $globalError = false;
    }

    $configOptions[$selected][2]($lineTab, $configurationParsing);

    if ($quietMode == true) {
      /* Check if error flag is set or one: Had an error occured ? */
      if ($globalError == true) {
	$output = ob_get_clean();
      }
      ob_clean();
      if ($globalError == true) {
	/* We need to display the error so we stop the buffering */
	ob_end_clean();
	echo $output;
	/* Restart the buffering */
	ob_start();
      }
    }
  }

  if ($configurationParsing) {
    if ($rbackupRoot == null) {
      echo "option rbackup_root missing in configuration file, I quit..\n";
      exit();
    }
    checkDestinationDirectory($rbackupRoot);

    
# Once we now the configuration is fine, we can rock!
    echo "[i] Starting rbackup with class `$currentClassName'\n";
  }

  if (! $configurationParsing) {
    break;
  }
}


function xsystem($cmd)
{
  system($cmd . ' 2>&1', $retValue);
  return $retValue;
}



function set_class($params, $configurationParsing)
{
  global $className;

  if ($configurationParsing) {
    return;
  }

  $className = $params[1];
}

function set_rbackup_root($params)
{
  global $rbackupRoot;
  $rbackupRoot = $params[1];
  if ($rbackupRoot[strlen($rbackupRoot) - 1] != '/') {
    $rbackupRoot .= '/';
  }
}

function checkDestinationDirectory($directory)
{
  if (! file_exists($directory)) {
    $directoryTab = explode('/', $directory);
    $subDirectory = '';
    for($i = 1; $i < sizeof($directoryTab) - 1; $i++) {
      $subDirectory .= '/' . $directoryTab[$i];
    }
    checkDestinationDirectory($subDirectory);
    @mkdir($directory) or die("cannot create directory $directory\n");
  }
}

function backup($params, $configurationParsing)
{
  global $rbackupRoot, $verboseMode, $className, $currentClassName;
  global $globalError;
  
  if ($configurationParsing) {
    return;
  }
  $sourceDirectory = $params[1];
  $destinationDirectory = $rbackupRoot . $params[2];

  $commandTab = array("rdiff-backup",
		      "-v3",
		      "--print-statistics",
		      "--create-full-path",
		      "--exclude-device-files",
		      "--exclude-other-filesystems",
		      "--preserve-numerical-ids",
#		      "--no-compression",
		      $sourceDirectory,
		      $destinationDirectory);
  $command = implode(' ', $commandTab);

  if ($className == null) {
    $className = DEFAULT_CLASS_NAME;
  }
  if ($currentClassName == $className) {
    echo "[b] $sourceDirectory -> $destinationDirectory\n";
    $retvalue = xsystem($command);

    if ($retvalue != 0) {
      $globalError = true;
      echo $commandTab[0], ":error! return value equal to ", $retvalue, "\n"; 
    }
  }
}

function mktemp($length)
{
  $chars = "abcdefghijklmnopqrstuvwxyz0123456789";
  $charsList = preg_split('//', $chars, -1, PREG_SPLIT_NO_EMPTY);
  do
    {
      for ($randomString = '', $i = 0; $i < $length; $i++) {
	$randomString .= $charsList[mt_rand() % 36];
      }
    } while ((@mkdir('/tmp/' . $randomString, 0700)) == false);
  return '/tmp/' . $randomString;
}

function svnsync($params, $configurationParsing)
{
  global $className, $currentClassName, $rbackupRoot, $verboseMode;

  if ($className == null) {
    $className = DEFAULT_CLASS_NAME;
  }
  if ($currentClassName != $className) {
    return;
  }

  if ($configurationParsing) {
    //    pr($params);
    return;
  }

  $repository = $params[1];
  $localDest = $rbackupRoot . $params[2];

  if (! file_exists($localDest)) {
    echo "[mkdir] creating directory $localDest\n";
    checkDestinationDirectory($localDest);

    echo "[svnadmin create] creating repository: $localDest\n";
    $commandTab = array('svnadmin', 'create', $localDest);
    $command = implode(" ", $commandTab);
    xsystem($command);
    
    echo "[svn hooks] changing hooks for synchronization\n";
    $hooksDir = $localDest . '/hooks/';
    $file1 = $hooksDir . 'start-commit';
    $file2 = $hooksDir . 'pre-revprop-change';
    $string = "#! /bin/sh\nexit 0\n";
    $fp = fopen($file1, "w");
    fwrite($fp, $string);
    fclose($fp);
    $commandTab = array('chmod', 'a+x', $file1);
    xsystem(implode(" ", $commandTab));
    $fp = fopen($file2, "w");
    fwrite($fp, $string);
    fclose($fp);
    $commandTab = array('chmod', 'a+x', $file2);
    xsystem(implode(" ", $commandTab));

    echo "[svnsync init] $repository -> $localDest\n";
    $commandTab = array('svnsync',
			'init',
			'file://' . $localDest,
			$repository);
    $command = implode(' ', $commandTab);
    xsystem($command);
  }

  echo "[svnsync] $repository -> $localDest\n";
  $commandTab = array('svnsync', 'sync', 'file://' . $localDest);
  $command = implode(' ', $commandTab);
  xsystem($command);
}

function script($params, $configurationParsing)
{
  global $className, $currentClassName, $rbackupRoot, $verboseMode;
  global $globalError;

  if ($configurationParsing) {
    return;
  }
  $scriptPathArgs = explode(',', $params[1]);
  $scriptPath = implode(' ', $scriptPathArgs);

  if ($className == null) {
    $className = DEFAULT_CLASS_NAME;
  }
  if ($currentClassName == $className) {
    $destinationDirectory = $rbackupRoot . $params[2];
    $tempDirectory = mktemp(10);
    echo "[chdir] going in $tempDirectory\n";
    chdir($tempDirectory);

    echo "[script] $scriptPath\n";
    $retValue = xsystem($scriptPath);
    if ($retValue != 0) {
      $globalError = true;
      echo "Error! $scriptPath returns $retValue\n";
      return;
    }

    $commandTab = array("rdiff-backup",
			"-v3",
			"--print-statistics",
			"--create-full-path",
			"--exclude-device-files",
			"--exclude-other-filesystems",
		      "--preserve-numerical-ids",
#		      "--no-compression",
			$tempDirectory,
			$destinationDirectory);
    $command = implode(' ', $commandTab);
    echo "[rdiff-backup] $tempDirectory -> $destinationDirectory\n";
    if ($verboseMode) {
      echo "[debug] $command\n";
    }
    xsystem($command);

    xsystem("rm -fr $tempDirectory");
  }
}

function syntax($argv)
{
  $progname = $argv[0];
  $len = strlen($progname);
  $zC = "%-3s";
  $fC = "%-" . $len . "s";
  $sC = "%-" . 21 . "s";
  
  echo "Syntax:\n";
  printf("$zC$fC -c $sC (default: %s)\n",
	 null, $progname, '[configuration_file]', DEFAULT_CONFIG_FILE);
  printf("$zC$fC -t $sC set the class to backup, default: %s\n",
	 null, null, "[class_name]", DEFAULT_CLASS_NAME);
  printf("$zC$fC -v $sC verbose mode\n", null, null, null);
  printf("$zC$fC -h $sC show this help\n", null, null, null);
}

ob_end_clean();
