#! /usr/bin/php -q
<?php
/*
bin=/usr/local/mysql-4.1.20/bin/mysqldump
$bin -u root --password='XXX' --add-drop-table \
    --complete-insert --default-character-set=latin1 --comments \
    --add-drop-database --create-options --hex-blob \
    --opt --quote-names --skip-extended-insert \
    --databases $@
*/

define('DEFAULT_HOST_INFORMATION', 'toad@217.73.17.12');

ini_set('memory_limit', '-1');
$socketPath = null;
$database = null;
$hostInformation = null;
$localDirectory = getenv("PWD");
$remoteDirectory = null;
$startingTime = 0;
$mysqlUsername = null;
$mysqlPassword = null;
$noArgument = true;

$options = getopt("s:d:h:u:p:");
foreach ($options as $option => $value) {
  $noArgument = false;
  switch ($option)
    {
    case 's':
      $socketPath = $value;
      break;
    case 'd':
      $database = $value;
      break;
    case 'h':
      $hostInformation = $value;
      break;
    case 'u':
      $mysqlUsername = $value;
      break;
    case 'p':
      $mysqlPassword = $value;
      break;
    }
}

if ($noArgument) {
  echo "rBackup MySQL dump\n";
  echo "Syntax: ./rbackup_mysql -h  this help\n";
  echo "                        -s  unix socket path\n";
  echo "                        -d  database name\n";
  echo "                        -h  host information to be used by SSH\n";
  echo "                            (i.e.: toad@lescigales.org)\n";
  echo "                        -u  username\n";
  echo "                        -p  password\n";
  exit(0);
}

$mysqlDumpTab = array('mysqldump',
		      "--add-drop-table",
		      "--add-drop-database",
		      "--complete-insert",
		      "--create-options",
		      "--comments",
		      "--opt",
		      "--quote-names",
		      "--skip-extended-insert"
		      );

if ($mysqlUsername != null) {
  array_push($mysqlDumpTab, "-u $mysqlUsername");
}
if ($mysqlPassword != null) {
  array_push($mysqlDumpTab, "--password='$mysqlPassword'");
}
if ($socketPath != null) {
  array_push($mysqlDumpTab, "-S $socketPath");
}

if ($database == null) {
  echo "no database specified\n";
  exit(-1);
} else {
  array_push($mysqlDumpTab, "--databases $database");
}
if ($hostInformation == null) {
  $hostInformation = DEFAULT_HOST_INFORMATION;
}
$mysqlDump = implode(' ', $mysqlDumpTab);

echo "MySQL rBackup. backing up database `$database'\n";

function genCommand($cmd) {
  global $hostInformation;
  return 'ssh '. $hostInformation . ' "'. $cmd . '"';
}

function diffTime() {
  global $startingTime;
  return time() - $startingTime . " secs.";
}

# Create a temporary directory on the server
exec(genCommand("mktemp -d"), $out, $retValue);
if ($retValue != 0) { echo "Error! getting out of here..\n"; exit(-1); }
$remoteDirectory = $out[0];
echo "[mktemp] remoteDirectory: $remoteDirectory\n";

# Dump a whole database in the remote directory
$startingTime = time();
$mysqlDumpCommand = genCommand("$mysqlDump > $remoteDirectory/$database.sql");
echo "[mysqldump] dumping database $database.sql\n";
system($mysqlDumpCommand, $retValue);
if ($retValue != 0) { echo "Error! getting out of here..\n"; exit(-1); }
echo "[mysqldump] export over in ", diffTime(), "\n";

# Compress it
$startingTime = time();
$gzipCmd = genCommand("gzip --fast $remoteDirectory/$database.sql");
echo "[gzip] $gzipCmd\n";
system($gzipCmd, $retValue);
if ($retValue != 0) { echo "Error! getting out of here..\n"; exit(-1); }
echo "[gzip] compression done in ", diffTime(), "\n";

# Fetch it locally
$startingTime = time();
$scpCmd = "scp $hostInformation:$remoteDirectory/$database.sql.gz $localDirectory";
echo "[scp] $scpCmd\n";
system($scpCmd, $retValue);
if ($retValue != 0) { echo "Error! getting out of here..\n"; exit(-1); }
echo "[scp] copied over the network in ", diffTime(), "\n";

# Remove the server's directory
$rmCmd = genCommand("rm -fr $remoteDirectory");
echo "[rm] $rmCmd\n";
system($rmCmd, $retValue);
if ($retValue != 0) { echo "Error! getting out of here..\n"; exit(-1); }

# Uncompress
$startingTime = time();
$ungzipCmd = "gzip -d $localDirectory/$database.sql.gz";
echo "[gzip] $ungzipCmd\n";
system($ungzipCmd, $retValue);
if ($retValue != 0) { echo "Error! getting out of here..\n"; exit(-1); }
echo "[gzip] archive decompressed in ", diffTime(), "\n";

return 0;
